--[[

Midpoint loop thingy (rough normalisation)

obey.

																		]]
--======================================================================--
-- Shortcuts/Libraries													--
--======================================================================--
gfx = love.graphics
lfs = love.filesystem
li = love.image
keydown = love.keyboard.isDown
audio = love.audio
mouse = love.mouse
mousedown = love.mouse.isDown
function sin(x) return math.sin(math.rad(x)) end
function cos(x) return math.cos(math.rad(x)) end
function tan(x) return math.tan(math.rad(x)) end
function asin(x) return math.deg(math.asin(x)) end
function acos(x) return math.deg(math.acos(x)) end
function atan(x) return math.deg(math.atan(x)) end
require "extra"
--======================================================================--
-- The 'Big Three' - load, update, draw									--
--======================================================================--
function love.load()
-------- System stuff
	resolutions = gfx.getModes()
	table.sort(resolutions, function(a,b) return a.width*a.height < b.width*b.height end)   -- sort from smallest to largest
	screen_w,screen_h,fullscreen,vsync,fsaa = gfx.getMode()
	gfx.setBackgroundColor(60,60,60)
-------- Resources
	fontsize_def = 15
	font_def = gfx.newFont(fontsize_def)
-------- Setup values
	toggle_debug = true
	toggle_controls = true
	no_points = 200
	reset()
	xmax = 0
	xmin = screen_w
	ymax = 0
	ymin = screen_h
--------
end
--------------------------------------------------------------------------
function love.update(dt)
-------- FPS limiter
	if dt < 1/60 then
		love.timer.sleep(1/60 - dt)
	end
	if keydown(" ") then
		iterate()
	end
	if keydown("up") then no_points = no_points + 1 end
	if keydown("down") then no_points = no_points - 1 end

	xmax = 0
	xmin = screen_w
	ymax = 0
	ymin = screen_h
	for i=1,#points do
		if points[i][1] < xmin then xmin = points[i][1] end
		if points[i][1] > xmax then xmax = points[i][1] end
		if points[i][2] < ymin then ymin = points[i][2] end
		if points[i][2] > ymax then ymax = points[i][2] end
	end
end
--------------------------------------------------------------------------
function love.draw()
	gfx.push()
	gfx.scale(screen_w/(xmax-xmin),screen_h/(ymax-ymin))
	gfx.translate(-xmin,-ymin)
	gfx.setColor(color_white)
	for i=1,#points do
		if i == #points then
			gfx.line(points[i][1],points[i][2],points[1][1],points[1][2])
		else
			gfx.line(points[i][1],points[i][2],points[i+1][1],points[i+1][2])
		end
	end
	gfx.pop()
	gfx.setFont(font_def)
	if toggle_controls == true then
		gfx.setColor(0,0,0,200)
		gfx.rectangle("fill",200,0,300,6*fontsize_def)
		gfx.setColor(color_white)
		gfx.print("` - toggle debug mode",200,fontsize_def*1-fontsize_def)	
		gfx.print("i - toggle this message",200,fontsize_def*2-fontsize_def)	
		gfx.print("hold space - iterate",200,fontsize_def*3-fontsize_def)	
		gfx.print("n - new loop",200,fontsize_def*4-fontsize_def)	
		gfx.print("up - increase number of points",200,fontsize_def*5-fontsize_def)	
		gfx.print("down - decrease number of points",200,fontsize_def*6-fontsize_def)	
	end
-------- Debug mode items to display (SHOULD BE DRAWN LAST)
	gfx.print("Debug mode: "..tostring(toggle_debug),0,fontsize_def*1-fontsize_def)
	if toggle_debug == true then
		gfx.setColor(0,0,0,200)
		gfx.rectangle("fill",0,0,190,7*fontsize_def)
		gfx.setColor(color_white)
		gfx.print("Debug mode: "..tostring(toggle_debug),0,fontsize_def*1-fontsize_def)
		gfx.print("no of points: "..no_points,0,fontsize_def*2-fontsize_def)
		gfx.print("iteration: "..iteration,0,fontsize_def*3-fontsize_def)
		gfx.print("minimum x coord: ~"..math.floor(xmin),0,fontsize_def*4-fontsize_def)		
		gfx.print("maximum x coord: ~"..math.floor(xmax),0,fontsize_def*5-fontsize_def)		
		gfx.print("minimum x coord: ~"..math.floor(ymin),0,fontsize_def*6-fontsize_def)		
		gfx.print("maximum y coord: ~"..math.floor(ymin),0,fontsize_def*7-fontsize_def)		
	end
end
--======================================================================--
-- Everything else														--
--======================================================================--
function love.keypressed(key)
--------
	if key == "`" then
		toggle_debug = not toggle_debug
	end
	if key == "i" then
		toggle_controls = not toggle_controls
	end
	if key == "n" then
		reset()
	end
--------
end
--------------------------------------------------------------------------
function wait(seconds)
	while seconds > 0 do
		seconds = seconds - coroutine.yield(true)
	end
end
--------------------------------------------------------------------------
function iterate()
	for i=1,iter_no do
		for i=1,#points do
			local x = 0
			local y = 0
			if i == #points then
				x = ((points[1][1] - points[i][1])/2) + points[i][1]
				y = ((points[1][2] - points[i][2])/2) + points[i][2]
			else
				x = ((points[i+1][1] - points[i][1])/2) + points[i][1]
				y = ((points[i+1][2] - points[i][2])/2) + points[i][2]
			end
			points[i][1] = x
			points[i][2] = y
		end
		iteration = iteration + 1
	end
	iter_no = iter_no + 1
end
--------------------------------------------------------------------------
function reset()
	iteration = 0
	points = {}
	for i=1,no_points do
		local x = math.random(0,screen_w)
		local y = math.random(0,screen_h)
		local a = {x,y}
		table.insert(points,a)
		iter_no = 1
	end
end
--------------------------------------------------------------------------
--------------------------------------------------------------------------
--------------------------------------------------------------------------
--------------------------------------------------------------------------